﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;
using System.Data.OleDb;

namespace copySheetSample
{
    public partial class Form1 : Form
    {
        int tableId;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            System.IO.File.Copy( @"sample.xlsx", @"copied.xlsx", true );

            copySheet(@"copied.xlsx", "Sheet1", "Sheet2");

            string connectString = "Provider=Microsoft.ACE.OLEDB.12.0; Data Source="
                                        + @"copied.xlsx;"
                                        + " Extended Properties=\"Excel 12.0; HDR=No;\"";

            using( OleDbConnection oleConnection = new OleDbConnection( connectString ) )
            {
                oleConnection.Open();

                OleDbCommand command = new OleDbCommand( "update [Sheet1$] set F1 = @a where F1 = @b", oleConnection );
                command.Parameters.Add( "@a", OleDbType.Char, 10 ).Value = "かきくけこ";
                command.Parameters.Add( "@b", OleDbType.Char, 10 ).Value = "あいうえお";
                command.ExecuteNonQuery();

                command = new OleDbCommand("update [Sheet2$] set F1 = @a where F1 = @b", oleConnection );
                command.Parameters.Add( "@a", OleDbType.Char, 10 ).Value = "かきくけこ";
                command.Parameters.Add( "@b", OleDbType.Char, 10 ).Value = "あいうえお";
                command.ExecuteNonQuery();

                oleConnection.Close();
            }
        
        
        }

        private void copySheet(string fileName, string sheetName, string clonedSheetName)
        {
            // Open workbook
            using (SpreadsheetDocument mySpreadsheet = SpreadsheetDocument.Open(fileName, true))
            {
                WorkbookPart workbookPart = mySpreadsheet.WorkbookPart;
                WorksheetPart sourceSheetPart = GetWorkSheetPart(workbookPart, sheetName);

                SpreadsheetDocument tempSheet = SpreadsheetDocument.Create(new MemoryStream(), mySpreadsheet.DocumentType);
                WorkbookPart tempWorkbookPart = tempSheet.AddWorkbookPart();
                WorksheetPart tempWorksheetPart = tempWorkbookPart.AddPart<WorksheetPart>(sourceSheetPart);
                WorksheetPart clonedSheet = workbookPart.AddPart<WorksheetPart>(tempWorksheetPart);

                int numTableDefParts = sourceSheetPart.GetPartsCountOfType<TableDefinitionPart>();
                tableId = numTableDefParts;
                if (numTableDefParts != 0)
                {
                    FixupTableParts(clonedSheet, numTableDefParts);
                }
                CleanView(clonedSheet);

                Sheets sheets = workbookPart.Workbook.GetFirstChild<Sheets>();
                Sheet copiedSheet = new Sheet();
                copiedSheet.Name = clonedSheetName;
                copiedSheet.Id = workbookPart.GetIdOfPart(clonedSheet);
                copiedSheet.SheetId = (uint)sheets.ChildElements.Count + 1;
                sheets.Append(copiedSheet);
                workbookPart.Workbook.Save();
                tempSheet.Close();
            }

        }

        private WorksheetPart GetWorkSheetPart(WorkbookPart workbookPart, string sheetName)
        {
            string relId = workbookPart.Workbook.Descendants<Sheet>().Where(s => s.Name.Value.Equals(sheetName)).First().Id;
            return (WorksheetPart)workbookPart.GetPartById(relId);
        }

        private void FixupTableParts(WorksheetPart worksheetPart, int numTableDefParts)
        {
            foreach (TableDefinitionPart tableDefPart in worksheetPart.TableDefinitionParts)
            {
                tableId++;
                tableDefPart.Table.Id = (uint)tableId;
                tableDefPart.Table.DisplayName = "CopiedDisplayTable" + tableId;
                tableDefPart.Table.Name = "CopiedTable" + tableId;
                tableDefPart.Table.Save();
            }
        }

        private void CleanView(WorksheetPart worksheetPart)
        {
            SheetViews views = worksheetPart.Worksheet.GetFirstChild<SheetViews>();
            if (views != null)
            {
                views.Remove();
                worksheetPart.Worksheet.Save();
            }
        }
    }
}
